package com.fjhx.client.impl;

import cn.hutool.core.util.ObjectUtil;
import com.fjhx.client.ClientAbstract;
import com.fjhx.entity.ConnectConfig;
import com.fjhx.entity.agreement.data.OpcUaData;
import lombok.SneakyThrows;
import org.eclipse.milo.opcua.sdk.client.api.identity.AnonymousProvider;
import org.eclipse.milo.opcua.sdk.client.nodes.UaNode;
import org.eclipse.milo.opcua.stack.core.Identifiers;
import org.eclipse.milo.opcua.stack.core.security.SecurityPolicy;
import org.eclipse.milo.opcua.stack.core.types.builtin.LocalizedText;
import org.eclipse.milo.opcua.stack.core.types.builtin.NodeId;
import org.eclipse.milo.opcua.stack.core.types.builtin.unsigned.UInteger;

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * opcUa链接实现
 */
public class OpcUaClient extends ClientAbstract {

    private org.eclipse.milo.opcua.sdk.client.OpcUaClient client;

    public OpcUaClient(ConnectConfig connectConfig) {
        super(connectConfig);
    }

    @SneakyThrows
    @Override
    public void connect() {
        Path securityTempDir = Paths.get(System.getProperty("java.io.tmpdir"), "security");
        Files.createDirectories(securityTempDir);
        if (!Files.exists(securityTempDir)) {
            throw new Exception("unable to create security dir: " + securityTempDir);
        }

        // 端口号
        String port = ObjectUtil.isEmpty(connectConfig.getPort()) ? "" : ":" + connectConfig.getPort();

        // 连接
        client = org.eclipse.milo.opcua.sdk.client.OpcUaClient.create(
                "opc.tcp://" + connectConfig.getIpAddress() + port,
                endpoints -> endpoints.stream()
                        .filter(e -> e.getSecurityPolicyUri().equals(SecurityPolicy.None.getUri()))
                        .findFirst(),
                configBuilder -> configBuilder
                        .setApplicationName(LocalizedText.english("eclipse milo opc-ua client"))
                        .setApplicationUri("urn:eclipse:milo:examples:client")
                        //访问方式
                        .setIdentityProvider(new AnonymousProvider())
                        .setRequestTimeout(UInteger.valueOf(5000))
                        .build());

        //开启连接
        client.connect().get();
    }

    @Override
    public Object readData() {
        List<OpcUaData> nodeList = new ArrayList<>();
        browseNode(nodeList, null);
        return nodeList;
    }

    /**
     * 遍历树形节点
     *
     * @param uaNode 节点
     */
    @SneakyThrows
    private void browseNode(List<OpcUaData> nodeList, UaNode uaNode) {
        List<? extends UaNode> nodes;

        if (uaNode == null) {
            nodes = client.getAddressSpace().browseNodes(Identifiers.ObjectsFolder);
        } else {
            nodes = client.getAddressSpace().browseNodes(uaNode);
        }

        for (UaNode nd : nodes) {
            // 排除系统行性节点，这些系统性节点名称一般都是以"_"开头
            if (Objects.requireNonNull(nd.getBrowseName().getName()).contains("_")) {
                continue;
            }

            NodeId nodeId = nd.getNodeId();

            OpcUaData opcUaData = new OpcUaData();
            opcUaData.setId(nodeId.getIdentifier().toString());
            opcUaData.setNs(nodeId.getNamespaceIndex().toString());
            opcUaData.setType(nodeId.getType().name());
            opcUaData.setNodeName(nd.getBrowseName().getName());
            nodeList.add(opcUaData);

            browseNode(nodeList, nd);
        }
    }

}
